@@ -17,7 +17,7 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import importlib
+import importlib, urllib.request, urllib.parse
 
 from owlready2.base import *
 from owlready2.base import _universal_abbrev_2_iri, _universal_iri_2_abbrev, _universal_abbrev_2_datatype, _universal_datatype_2_abbrev
@@ -334,7 +334,7 @@ WHERE q1.p=? AND q1.o=?
         sub = self._get_obj_triple_po_s(rdfs_subclassof, s)
         if sub is None: yield self._parse_bnode(s)
         
-  def search(self, _use_str_as_loc_str = True, _case_sensitive = True, **kargs):
+  def search(self, _use_str_as_loc_str = True, _case_sensitive = True, _bm25 = False, **kargs):
     from owlready2.triplelite import _SearchList, _SearchMixin
     
     prop_vals = []
@@ -343,7 +343,7 @@ WHERE q1.p=? AND q1.o=?
       for v in v0:
         if   k == "iri":
           prop_vals.append((" iri", v, None))
-        elif (k == "is_a") or (k == "subclass_of") or (k == "type"):
+        elif (k == "is_a") or (k == "subclass_of") or (k == "type") or (k == "subproperty_of"):
           if   isinstance(v, (_SearchMixin, Or)): v2 = v
           elif isinstance(v, int):                v2 = v
           else:                                   v2 = v.storid
@@ -352,7 +352,7 @@ WHERE q1.p=? AND q1.o=?
           d = None
           Prop = self.world._props.get(k)
           if Prop is None:
-            k2 = _universal_iri_2_abbrev.get(k) or k
+            k2 = _universal_iri_2_abbrev.get(k) or self.world._abbreviate(k, create_if_missing = False) or k
           else:
             if Prop.inverse_property:
               k2 = (Prop.storid, Prop.inverse.storid)
@@ -375,7 +375,7 @@ WHERE q1.p=? AND q1.o=?
                 
           prop_vals.append((k2, v2, d))
           
-    return _SearchList(self.world, prop_vals, None, _case_sensitive)
+    return _SearchList(self.world, prop_vals, None, _case_sensitive, _bm25)
     
   def search_one(self, **kargs): return self.search(**kargs).first()
   
@@ -413,7 +413,7 @@ def _clear_cache():
   for i in d.keys():
     pass
 
-
+WORLDS = weakref.WeakSet()
 class World(_GraphManager):
   def __init__(self, backend = "sqlite", filename = ":memory:", dbname = "owlready2_quadstore", **kargs):
     global owl_world
@@ -431,6 +431,7 @@ class World(_GraphManager):
     if not owl_world is None:
       self._entities.update(owl_world._entities) # add OWL entities in the world
       self._props.update(owl_world._props)
+      WORLDS.add(self)
       
     if filename:
       self.set_backend(backend, filename, dbname, **kargs)
@@ -557,7 +558,7 @@ class World(_GraphManager):
             Class = self._get_by_storid(obj, None, ThingClass, main_onto)
             if isinstance(Class, EntityClass): types.append(Class)
             elif Class is None: raise ValueError("Cannot get '%s'!" % obj)
-
+            
       if main_type is None: # Try to guess it
         if   self._has_obj_triple_spo(None, rdf_type, storid) or self._has_obj_triple_spo(None, rdfs_subclassof, storid) or self._has_obj_triple_spo(storid, rdfs_subclassof, None): main_type = ThingClass
         elif self._has_obj_triple_spo(storid, None, None) or self._has_data_triple_spod(storid, None, None, None): main_type = Thing
@@ -667,7 +668,6 @@ class World(_GraphManager):
       for onto in self.ontologies.values():
         if onto.graph.c == c: return onto._parse_bnode(bnode)
       
-      
      
 class Ontology(Namespace, _GraphManager):
   def __init__(self, world, base_iri, name = None):
@@ -833,12 +833,21 @@ class Ontology(Namespace, _GraphManager):
     return self
   
   def _load_properties(self):
+    # Update props from other ontologies, if needed
+    for prop in list(self.world._props.values()):
+      if prop.namespace.world is owl_world: continue
+      if prop._check_update(self) and _LOG_LEVEL:
+        print("* Owlready2 * Reseting property %s: new triples are now available." % prop)
+        
+    # Loads new props
     props = []
-    #for i in self.graph.execute("select * from quads").fetchall(): print(i)
     for prop_storid in itertools.chain(self._get_obj_triples_po_s(rdf_type, owl_object_property), self._get_obj_triples_po_s(rdf_type, owl_data_property), self._get_obj_triples_po_s(rdf_type, owl_annotation_property)):
       Prop = self.world._get_by_storid(prop_storid)
       python_name_d = self.world._get_data_triple_sp_od(prop_storid, owlready_python_name)
       
+      if not isinstance(Prop, PropertyClass):
+        raise TypeError("'%s' belongs to more than one entity types (cannot be both a property and a class/an individual)!" % Prop.iri)
+      
       if python_name_d is None:
         props.append(Prop.python_name)
       else:
@@ -1136,7 +1145,7 @@ class Metadata(object):
   
 def _open_onto_file(base_iri, name, mode = "r", only_local = False):
   if base_iri.endswith("#") or base_iri.endswith("/"): base_iri = base_iri[:-1]
-  if base_iri.startswith("file://"): return open(base_iri[7:], mode)
+  if base_iri.startswith("file://"): return open(urllib.parse.unquote(base_iri[7:]), mode)
   for dir in onto_path:
     for ext in ["", ".owl", ".rdf", ".n3"]:
       filename = os.path.join(dir, "%s%s" % (name, ext))
@@ -1147,7 +1156,7 @@ def _open_onto_file(base_iri, name, mode = "r", only_local = False):
 
 def _get_onto_file(base_iri, name, mode = "r", only_local = False):
   if base_iri.endswith("#") or base_iri.endswith("/"): base_iri = base_iri[:-1]
-  if base_iri.startswith("file://"): return base_iri[7:]
+  if base_iri.startswith("file://"): return urllib.parse.unquote(base_iri[7:])
   
   for dir in onto_path:
     filename = os.path.join(dir, base_iri.rsplit("/", 1)[-1])
@@ -1160,6 +1169,15 @@ def _get_onto_file(base_iri, name, mode = "r", only_local = False):
   raise FileNotFoundError
 
 
+# def convert_with_owlapi(orig, format = "nt"):
+#   import tempfile
+#   fileno, filename = tempfile.mkstemp()
+#   command = [owlready2.JAVA_EXE, "-cp", owlready2.reasoning._HERMIT_CLASSPATH, "Save", orig, format, filename]
+#   print(" ".join(command))
+#   output = subprocess.check_output(command, stderr = subprocess.STDOUT)
+#   return filename
+  
+
 
 owl_world = World(filename = None)
 rdf       = owl_world.get_ontology("http://www.w3.org/1999/02/22-rdf-syntax-ns#")
